﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace tetris.v4
{
  public class Memory
  {
    public enum pieceType
    {
      ELLE_DESTRA = 1,
      ELLE_SINISTRA = 2,
      BARRA = 3,
      MATTONE = 4,
      TI = 5,
      SFASATO_DESTRA = 6,
      SFASATO_SINISTRA = 7,
      LINEA_LAMPEGGIANTE_COL1 = 8,
      LINEA_LAMPEGGIANTE_COL2 = 9
    }
    public enum pieceRotation
    {
      NO = 1,
      NOVANTA = 2,
      CENTOOTTANTA = 3,
      DUEESETTANTA = 4
    }
    public const int SCREEN_LINES = 18;
    public const int SCREEN_COLUMNS = 9;
    public const int LEVEL_LAXT = 0;
    public const int LEVEL_FIRST = 9;
    public const int LEVEL_OFFSET = 10;
    public const int LEVEL_MILLISECOND_MULTIPLY  = 100;
    public const int PEACE_SIZE_X = 4;
    public const int PEACE_SIZE_Y = 4;
    public const int SCORE_PER_LINE = 100;


    public int[,] screen = new int[SCREEN_COLUMNS, SCREEN_LINES];
    public bool[,] currPiece = new bool[PEACE_SIZE_X, PEACE_SIZE_Y];
    public bool[,] currPieceAfterRotate = new bool[PEACE_SIZE_X, PEACE_SIZE_Y];
    public pieceType currPieceType = pieceType.MATTONE;
    public pieceRotation currPieceRotation = pieceRotation.NO;
    public bool[,] next = new bool[PEACE_SIZE_X, PEACE_SIZE_Y];
    public pieceType nextType= pieceType.MATTONE;

    public int currLevel = LEVEL_FIRST;
    public int currScore = 0;
    public int currLines = 0;
    public int currX = 0;
    public int currY = 0;

    public bool[,] getPiece(pieceType type, pieceRotation angle)
    {
      bool[,] retArray = new bool[PEACE_SIZE_X, PEACE_SIZE_Y];

      switch (type)
      {
        case pieceType.ELLE_DESTRA:
          switch (angle)
          {
            case pieceRotation.NO:
              /*
               0  # 
               1###
               2
               3
               *0123
               */
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              retArray[2, 1] = true;
              retArray[2, 0] = true;
              break;
            case pieceRotation.NOVANTA:
              /*
               0##
               1 #
               2 # 
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[1, 1] = true;
              retArray[1, 2] = true;
              break;
            case pieceRotation.CENTOOTTANTA:
              /*
               0###
               1#
               2
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[2, 0] = true;
              retArray[0, 1] = true;
              break;
            case pieceRotation.DUEESETTANTA:
              /*
               0#
               1#
               2##
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[0, 1] = true;
              retArray[0, 2] = true;
              retArray[1, 2] = true;
              break;
          }
          break;



        case pieceType.ELLE_SINISTRA:
          switch (angle)
          {
            case pieceRotation.NO:
              /*
               0# 
               1###
               2 
               3
               *0123
               */
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              retArray[2, 1] = true;
              retArray[0, 0] = true;
              break;
            case pieceRotation.NOVANTA:
              /*
               0 #
               1 #
               2## 
               3
               *0123
               */
              retArray[1, 0] = true;
              retArray[1, 1] = true;
              retArray[1, 2] = true;
              retArray[0, 2] = true;
              break;
            case pieceRotation.CENTOOTTANTA:
              /*
               0### 
               1  #
               2
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[2, 0] = true;
              retArray[2, 1] = true;
              break;
            case pieceRotation.DUEESETTANTA:
              /*
               0## 
               1#
               2# 
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[0, 1] = true;
              retArray[0, 2] = true;
              break;
          }
          break;




        case pieceType.BARRA:
          switch (angle)
          {
            case pieceRotation.NO:
            case pieceRotation.CENTOOTTANTA:
              /*
               0#### 
               1
               2
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[2, 0] = true;
              retArray[3, 0] = true;
              break;
            case pieceRotation.NOVANTA:
            case pieceRotation.DUEESETTANTA:
              /*
               0#
               1#
               2#
               3#
               *0123
               */
              retArray[0, 0] = true;
              retArray[0, 1] = true;
              retArray[0, 2] = true;
              retArray[0, 3] = true;
              break;
          }
          break;
    
        
        
        case pieceType.MATTONE:
          retArray[1, 0] = true;
          retArray[2, 0] = true;
          retArray[1, 1] = true;
          retArray[2, 1] = true;
          break;
        
        
        
        case pieceType.TI:
          switch (angle)
          {
            case pieceRotation.NO:
              /*
               0 # 
               1###
               2 
               3
               *0123
               */
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              retArray[2, 1] = true;
              retArray[1, 0] = true;
              break;
            case pieceRotation.NOVANTA:
              /*
               0 #
               1##
               2 #
               3
               *0123
               */
              retArray[1, 0] = true;
              retArray[1, 1] = true;
              retArray[1, 2] = true;
              retArray[0, 1] = true;
              break;
            case pieceRotation.CENTOOTTANTA:
              /*
               0###
               1 #
               2 
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[2, 0] = true;
              retArray[1, 1] = true;
              break;
            case pieceRotation.DUEESETTANTA:
              /*
               0# 
               1##
               2# 
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[0, 1] = true;
              retArray[0, 2] = true;
              retArray[1, 1] = true;
              break;
          }
          break;

        
        
        case pieceType.SFASATO_DESTRA:
          switch (angle)
          {
            case pieceRotation.NO:
            case pieceRotation.CENTOOTTANTA:
              /*
               0 ## 
               1##
               2 
               3
               *0123
               */
              retArray[1, 0] = true;
              retArray[2, 0] = true;
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              break;
            case pieceRotation.NOVANTA:
            case pieceRotation.DUEESETTANTA:
              /*
               0# 
               1##
               2 #
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              retArray[1, 2] = true;
              break;
          }
          break;
        
        
        case pieceType.SFASATO_SINISTRA:
          switch (angle)
          {
            case pieceRotation.NO:
            case pieceRotation.CENTOOTTANTA:
              /*
               0## 
               1 ##
               2 
               3
               *0123
               */
              retArray[0, 0] = true;
              retArray[1, 0] = true;
              retArray[1, 1] = true;
              retArray[2, 1] = true;
              break;
            case pieceRotation.NOVANTA:
            case pieceRotation.DUEESETTANTA:
              /*
               0 #
               1##
               2#
               3
               *0123
               */
              retArray[1, 0] = true;
              retArray[0, 1] = true;
              retArray[1, 1] = true;
              retArray[0, 2] = true;
              break;
          }
          break;
      }
      return retArray;
    }

    public pieceType getRandomPiece()
    {
      Random r = new Random();
      int rNumb = r.Next(1, 7900);
      int toPieceTypeRNumb = (int)rNumb / 1000;
      if (toPieceTypeRNumb <= 0)
      {
        toPieceTypeRNumb = 1;
      }
      if (toPieceTypeRNumb > 7)
      {
        toPieceTypeRNumb = 7;
      }
      return (pieceType)toPieceTypeRNumb;
     
    }

    public Color pieceType2Color(pieceType type)
    {
      switch (type)
      {
        case pieceType.ELLE_DESTRA:
          return Color.DarkGreen;
        case pieceType.ELLE_SINISTRA:
          return Color.Yellow;
        case pieceType.BARRA:
          return Color.Red;
        case pieceType.MATTONE:
          return Color.Brown;
        case pieceType.TI:
          return Color.Orange;
        case pieceType.SFASATO_DESTRA:
          return Color.Lime;
        case pieceType.SFASATO_SINISTRA:
          return Color.Magenta;
        case pieceType.LINEA_LAMPEGGIANTE_COL1:
          return Color.White;
        case pieceType.LINEA_LAMPEGGIANTE_COL2:
          return Color.Red;
      }
      return Color.White;
    }
    
  }
}
